function [T,R,W] = bidiagFULL(X)
%BIDIAGFULL
% Upper bidiagonalization of an m-times-n matrix.
% Full incusing nullspace vectors -- not 'economy size'.
% [T,R,W] = bidiagFULL(X)

% Reference: L. Elden, "Algorithms for regularization of ill-
% conditioned least-squares problems", BIT 17 (1977), 134-145.
% Original version, Per Christian Hansen, UNI-C, 03/11/92.
% 2010/08/11, handle any size of A, Martin Andersson, Japan.

Xorig = X; % only for display purposes
[m,n] = size(X);
runTransposed = (m < n);
if runTransposed,
    X = X';
    [m,n] = size(X);
end

%X = [X zeros(m,m-n)];
R = [zeros(m,n)];
T = eye(m);
W = eye(n);
betaT = zeros(m,1);
betaW = zeros(n,1);

% UPPER Bidiagonalization; save Householder quantities.
for k=1:n,
    [R(k,k),betaT(k),X(k:m,k)] = gen_hh(X(k:m,k));
    X(k:m,k+1:n) = app_hh(X(k:m,k+1:n),betaT(k),X(k:m,k));
    if (k < n-1)
        [R(k,k+1),betaW(k),X(k,k+1:n)] = gen_hh(X(k,k+1:n));
        X(k+1:m,k+1:n) = app_hh(X(k+1:m,k+1:n)',betaW(k),X(k,k+1:n)')';
    elseif (k == n-1)
        R(n-1,n) = X(n-1,n);
    end
end

%fprintf('\nFrom bidiagNEW:'); A, AA=A; save AA AA;
% Compute U
for k=n:-1:1,
    %fprintf('A(%i:m,%i) = [ ',k,k)
    %fprintf('%g ',A(k:m,k))
    %fprintf('   ]\n')
    T(k:m,k:m) = app_hh(T(k:m,k:m),betaT(k),X(k:m,k));
    %fprintf('U(%i:m,%i:n) =\n',k,k); U(k:m,k:n)
end
% Compute V
for k=n-2:-1:1
    W(k+1:n,k:n) = app_hh(W(k+1:n,k:n),betaW(k),X(k,k+1:n)');
    %[k+1 n]-1, A(k,k+1:n)
    %fprintf('\nV =\n\n',k+1,k); disp(V)
end
%fprintf('betaV = [ ');
%fprintf('%g ',betaV);
%fprintf(']\nbetaU =[ ');
%fprintf('%g ',betaU);
%fprintf(']\n\n');

if runTransposed,
    % flip (used here) or use permutation matrices
    oldW = W;
    W = [fliplr(T(:,1:n)) T(:,n+1:m)];
    R = [fliplr(flipud(R(1:n,1:n)')) zeros(n,m-n)];
    T = fliplr(oldW);
end
%disp(['|| U''U    -    I || = ',num2str(ssq(U'*U-eye(size(B))))]);
%disp(['|| V''V    -    I || = ',num2str(ssq(V'*V-eye(size(B))))]);
%disp(['|| x    - U B V'' || = ',num2str(ssq(Aorig-U*B*V'))]);
%disp(['||svd(A) - svd(B)|| = ',num2str(ssq(svd(Aorig)-svd(B)))]);

function [x1,beta,v] = gen_hh(x)
%GEN_HH Generate a Householder transformation.
%
% [x1,beta,v] = gen_hh(x)
% Given a vector x, gen_hh computes the scalar beta and the vector v
% determining a Householder transformation
%
% H = (I - beta*v*v'),
% such that H*x = +-norm(x)*e_1. x1 is the first element of H*x.
% Based on 'house' from Nicholas J. Higham's Test Matrix Toolbox.
% Per Christian Hansen, IMM, Sept. 13, 2001.
alpha = norm(x)*(sign(x(1)) + (x(1)==0));
v = x;
if (alpha==0),
    beta = 0;
else
    beta = 1/(abs(alpha)*(abs(alpha) + abs(v(1))));
end
%disp(['v(1) = ',num2str(v(1))])
v(1) = v(1) + alpha;
x1 = -alpha;

function A = app_hh(A,beta,v)
%APP_HH Apply a Householder transformation.
%
% A = app_hh(A,beta,v)
% Applies the Householder transformation, defined by
% vector v and scaler beta, to the matrix A; i.e.
%
% A = (eye - beta*v*v')*A .
% Per Christian Hansen, IMM, 03/11/92.
A = A - beta*(v*(v'*A));

